#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Oct 27 17:16:54 2019

@author: François Pelletier
"""

import nltk
import re
from collections import defaultdict

## Normalisation
    
def tokenize(reviews):
    tokenizer = nltk.tokenize.ToktokTokenizer()
    tokenized_reviews = []
    for review in reviews:
        # Plusieurs fin de phrases étaient représentées par deux espaces ou plus.
        review = re.sub(r"\s{2,}",". ",review)
        review = str.lower(review)
        tokenized_sentences = []
        sentences = nltk.sent_tokenize(review)
        for sentence in sentences:
            sentence_tokens = tokenizer.tokenize(sentence)
            # Suppression de la ponctuation
            for token in sentence_tokens:
                if token in "?:!.,;":
                    sentence_tokens.remove(token)
            tokenized_sentences.append(sentence_tokens)
        tokenized_reviews.append(tokenized_sentences)
    return tokenized_reviews

def norm_stemming(tokenized_reviews):
    porter = nltk.PorterStemmer()
    stemmed_reviews = []
    for review in tokenized_reviews:
        stemmed_review = []
        for sentence in review:
            stemmed_tokens = [porter.stem(token) for token in sentence]
            stemmed_review.append(stemmed_tokens)
        stemmed_reviews.append(stemmed_review)
    return stemmed_reviews

def norm_lemmatize(tokenized_reviews):
    wnl = nltk.WordNetLemmatizer()
    normalized_reviews = []
    for review in tokenized_reviews:
        normalized_review = []
        for sentence in review:
            # Lemmatisation au verbe et non au nom
            normalized_tokens = [wnl.lemmatize(token, pos="v") for token in sentence]
            normalized_review.append(normalized_tokens)
        normalized_reviews.append(normalized_review)
    return normalized_reviews

## Feature selection
    
def dict_frequency(norm_reviews):
    tokens_frequency = defaultdict(int)
    for review in norm_reviews:
        for sentence in review:
            for token in sentence:
                tokens_frequency[token] += 1
    return tokens_frequency

def select_freq(norm_reviews):
    tokens_frequency = dict_frequency(norm_reviews)
    norm_reviews_freq3 = [
     [
      [token for token in sentence if tokens_frequency[token] > 3] 
      for sentence in review
      ] 
     for review in norm_reviews
     ]
    return norm_reviews_freq3
    
def select_rem_stopwords(norm_reviews):
    sws = set(nltk.corpus.stopwords.words('english'))
    norm_reviews_stoprem = [
     [
      [token for token in sentence if token not in sws] 
      for sentence in review
      ] 
     for review in norm_reviews
     ]
    return norm_reviews_stoprem
    
def select_open_class(reviews):
    return 0
    
## Attribute value
    
def value_count(reviews):
    return 0
    
def value_occurence(reviews):
    return 0
    
def value_tfidf(reviews):
    return 0
    
## Other attributes
    
def attribute_polarity_count(reviews):
    return 0
    
def attribute_length(reviews):
    return 0
    
## Training
    
def train_naive_model(reviews):
    return 0
    
def train_regression_model(reviews):
    return 0
